function [Output,WRMSR] = SimulateAndFitIndirect(select_model,Knots,shapes,SFP,noise)

% SFP is a structure containing all relevant parameters for this function

Output = struct();

Nfun = zeros(1,length(select_model));
for i=1:length(select_model)
    Nfun(i) = find(strcmp(select_model{i},SFP.model_names));
end

Library	=	CreateLibrary	;
nLib    =   length(Library) ;

WRMSR = zeros(length(Nfun),nLib+length(Knots));

tspan           =	0:(.5/60):4      ;
tmeas           =   tspan(1:5:end)  ;

N = [ -1 +1];

iter = 1;

for iFun = Nfun
    
    
    % ====================================================================
    % 0. SIMULATE AND PROCESS DATA
    
    disp(['Case: ' num2str(iFun)])
    disp(['	Simulating rate function: ' SFP.model_names{iFun}])
    
    ratefun = SelectFunction(iFun,SFP.model_names);

    Output(iter).true_rate_name = SFP.model_names{iFun};
    Output(iter).true_rate_rate = ratefun(SFP.SS,SFP.pars);
    Output(iter).true_rate_SS = SFP.SS;
    
    % ==================================
    % Simulate
    [~,Ssim] 	= 	ode15s(@FittoRates_odefun,tspan,SFP.S0,SFP.odeoptions,'library',ratefun,SFP.pars)	;
    Csim        =   [   Ssim    Ssim(1)-Ssim    ]'       ;
    Cmeas       =       Csim(:,1:5:end)                 ;

    % ==================================
    % Add noise
    
    if exist('noise','var')==1
        Cmeas       =       Cmeas + noise                    ;
    else
        noise       =   randn(size(Cmeas))*SFP.sigmaY        ;
        Cmeas       =       Cmeas + noise                    ;
    end
    
    Output(iter).true_rate_ode = Cmeas;
    Output(iter).true_rate_ode_tmeas = tmeas;
    
    disp('     Simulation finished')
    
    % ==================================
    % Finite differentiation
    
    xhat        =	(N*N')^(-1)*N*Cmeas     ;
    [cc,index]	=	sort(Cmeas(1,2:end))    ;
    dxhat       =	diff(xhat)/(5/2/60)     ;
    dxhat       =	dxhat(index)            ;
    dxhat       =   dxhat(cc>0)             ;
    cc          =   cc(cc>0)                ;
    
    disp('     Simulation data processed')
    
    % ====================================================================
    % ====================================================================
    % I. CLASSIC FUNCTION LIBRARY
    
    disp('	Modeling via classic function library')
    
    Models = cell(1,nLib);
    Pars = cell(1,nLib);
    
    for iLib=1:nLib
        Model   =   Library(iLib) ;
        
        disp(['     Using model: ' Model.name])
        disp('         Initial parameter search')
        
        % ==================================
        % Initial guess via finite differentiation
        OptimPars_initialguess = FitLibrarytoRates_Optim(cc(:),dxhat(:),Model);
        disp('         Initial parameter search finished')
        
        % ==================================
        % Optimize actual objective function
        
        disp('         Nonlinear parameter optimization')
        
        options = optimoptions('fmincon');
        options.TolX = 1e-6;
        options.TolFun = 1e-6;
        options.Display='off';
        %options.LargeScale='off';
        
        OptimParsLib = fmincon(@(pars) EvaluateODE(tmeas,SFP.S0,Model.ratefun,Cmeas,pars,SFP,'library'),OptimPars_initialguess,[],[],[],[],zeros(size(Model.pars)),[],[],options);

        disp('         Nonlinear parameter optimization finished')
        
        Models{iLib}	=	Model       ; 
        Pars{iLib}      =   OptimParsLib	;
        
        [WRMSR(iter,iLib),ChatLib] = EvaluateODE(tmeas,SFP.S0,Model.ratefun,Cmeas,OptimParsLib,SFP,'library') ;
        
        Output(iter).Lib_models(iLib).name = Model.name;
        Output(iter).Lib_models(iLib).rate = Model.ratefun;
        Output(iter).Lib_models(iLib).pars = OptimParsLib;
        Output(iter).Lib_models(iLib).ode = ChatLib;
        Output(iter).Lib_models(iLib).tmeas = tmeas;
        
        % ==================================
               
    end

    disp('     Model selection')
    [~,IndexSelect]     =   min(WRMSR(iter,1:nLib))             ;
    ModelLibSelected    =	Models{IndexSelect} ;
    OptimParsSelected          =   Pars{IndexSelect}   ;
    disp(['     Library model selection result: ' ModelLibSelected.name])
    
    [~,ChatLibSelected] = EvaluateODE(tmeas,SFP.S0,ModelLibSelected.ratefun,Cmeas,OptimParsSelected,SFP,'library') ;
    
    Output(iter).Lib_models_selected.name = ModelLibSelected.name;
    Output(iter).Lib_models_selected.rate = ModelLibSelected.ratefun(SFP.SS,OptimParsSelected);
    Output(iter).Lib_models_selected.pars = OptimParsSelected;
    Output(iter).Lib_models_selected.ode = ChatLibSelected;
    Output(iter).Lib_models_selected.tmeas = tmeas;

    
    % ====================================================================
    % ====================================================================
    % II. SCS FUNCTION
    
    disp('	Modeling via SCS function')
    disp('     Initial parameter search')  
    disp('         Optimization')

    for k = 1:length(Knots)
        
        knots = Knots{k};
        shape = shapes{k};
        
        disp(['             Fitting using ' num2str(length(knots)) ' knots, shape ' shape '.'])
        
        [modelSCSinitial] = FitSCStoRates(cc,dxhat(:),'C',knots);

        % ==================================
        % Ensure guess is inside the feasible set:
        disp('              Off-stepping')

        options = optimoptions('fmincon');
        options.Display = 'final';
        options.TolX = 1e-6;
        options.TolFun = 1e-6;
        
        if strcmp(shape,'C') == 1

            A = [-modelSCSinitial.B1k(end,:) ;  modelSCSinitial.B2k] ;
        
        elseif strcmp(shape,'N') == 1

            A = [modelSCSinitial.B2k] ;

        else
            error('Unsupported shape constraint')        
        end
        
        %A = full(A) ;

        b = -ones(size(A,1),1)*1e-6 ;
        Aeq = []; 
        beq = []; 
        x0 = modelSCSinitial.x ;
        x1 = fmincon(@(x) sum((x-x0).^2),x0,A,b,Aeq,beq,[],[],[],options);
        modelSCSfeasible = modelSCSinitial;
        modelSCSfeasible.x = x1;

        disp('              Initial parameter search finished')

        % ==================================
        % Optimize actual objective function

        disp('              Nonlinear parameter optimization')

        b = -ones(size(A,1),1)*0 ;
        x2 = fmincon(@(x) EvaluateODE(tmeas,SFP.S0,modelSCSfeasible,Cmeas,[],SFP,'SCS',x),x1,A,b,Aeq,beq,[],[],[],options);
        modelSCSselected   =	modelSCSfeasible ;
        modelSCSselected.x =	x2;

        disp('              Nonlinear parameter optimization finished')
        
        [WRMSR(iter,nLib+k),ChatSCS] = EvaluateODE(tmeas,SFP.S0,modelSCSselected,Cmeas,[],SFP,'SCS') ;
        
        modelSCSselected.x =	x2*1.01;
        [WRMSR(iter,nLib+k),ChatSCS] = EvaluateODE(tmeas,SFP.S0,modelSCSselected,Cmeas,[],SFP,'SCS') ;
        
        
        Output(iter).SCS_models(k).rate = EvalDynSCS(modelSCSselected,SFP.SS,0);
        Output(iter).SCS_models(k).ode = ChatSCS;
        Output(iter).SCS_models(k).tmeas = tmeas;

    end
    
    iter = iter + 1;
    
end


